
        .MODEL  Small
        .DOSSEG                     ;DOS segment order
        .STACK  1024
        OPTION  CaseMap:None        ;case sensitive

;####################################################################

;equate(s)

BUFSIZE EQU 5                       ;user input buffer size, 255 max

;####################################################################

;structure(s)

INBUFFER  STRUCT
  MaxBytes  db ?
  UsedBytes db ?
  Buffer    db BUFSIZE+1 dup(?)
INBUFFER  ENDS

;####################################################################

        .DATA

s$Prompt1 db '   Enter the Number of Rows: ',24h
s$Prompt2 db 'Enter the Number of Columns: ',24h
s$ErrMsg1 db 'Zero Not Allowed',24h
s$ErrMsg2 db 'Non-Numeric Character in String',24h
s$ErrMsg3 db 'Value Too Large',24h
s$ErrMsg4 db 'Matrix Exceeds 65536 Bytes',24h
s$ErrMsg5 db 'Matrix Exceeds Available Memory',24h
s$AnyKey  db 0Dh,0Ah,'Press Any Key To Exit',24h
s$CrLf    db 0Dh,0Ah,24h

OutBuf    db 5 dup(?),24h

;********************************************************************

        .DATA?

nRows     dw ?
nColumns  dw ?

InpBuf    INBUFFER <>

;####################################################################

        .CODE

;********************************************************************

_main   PROC    FAR

;------------------------------------------

;set DS to DGROUP

        ASSUME  DS:DGROUP

        mov     ax,@data
        mov     ds,ax

;------------------------------------------

;initialize direction flag, input buffer size

;CLD may not be necessary however, it does not hurt to make sure
;we set InpBuf.MaxBytes in code so that the
;buffer may be in the uninitialized data segment

        cld
        mov    InpBuf.MaxBytes,BUFSIZE+1

;------------------------------------------

;get user input values

Input0: call    NewLine
        mov     dx,offset s$Prompt1 ;'Enter the Number of Rows:'
        call    PrInp
        mov     nRows,dx

        mov     dx,offset s$Prompt2 ;'Enter the Number of Columns:'
        call    PrInp
        mov     nColumns,dx

;validate user input

        mov     ax,nRows
        mul     dx
        or      dx,dx
        jnz     Input1

        shl     ax,1
        cmc
        jbe     Input3

Input1: mov     dx,offset s$ErrMsg4 ;'Matrix Exceeds 65536 Bytes'

Input2: call    StrOut
        jmp     Input0

;test available memory

Input3: dec     ax
        push    ax
        int     12h                 ;BIOS call - get available memory
        pop     di                  ;DI = array size in bytes, minus 1

        mov     cl,6
        shl     ax,cl               ;convert Kb to paragraphs

        mov     bx,sp
        add     bx,0Fh
        mov     cl,4
        shr     bx,cl

        mov     dx,ss
        add     bx,dx               ;BX = array segment

        mov     dx,di
        shr     dx,cl
        add     dx,bx
        cmp     ax,dx
        ja      Fill00

        mov     dx,offset s$ErrMsg5 ;'Matrix Exceeds Available Memory'
        jmp     Input2

;------------------------------------------

;fill the array: Matrix(X,Y) = X*Y

;BX = array segment
;DI = array size in bytes, minus 1

;we use SUB to step values rather than multiplying, as it's a bit faster

Fill00: call    NewLine
        mov     ax,di
        mov     es,bx               ;ES = array segment
        shr     ax,1
        dec     di                  ;DI = offset of last element
        inc     ax                  ;AX = X*Y product for last element
        mov     cx,nRows            ;CX = rows
        mov     bx,nColumns         ;BX = columns

Fill01: push    cx
        mov     dx,cx
        push    ax
        mov     cx,bx

Fill02: mov     es:[di],ax
        sub     di,2
        sub     ax,dx
        loop    Fill02

        pop     ax
        pop     cx
        sub     ax,bx
        loop    Fill01

;------------------------------------------

;display the results

;seems a little messy because we
;control line-wrap, commas, and spaces

        mov     si,nColumns
        mov     cx,nRows
        xor     di,di

Show00: push    si
        xor     bx,bx
        push    cx

Show01: mov     ax,es:[di]
        call    UnsDec
        xor     ax,ax
        or      bx,bx
        jz      Show02

        inc     cx
        inc     ax

Show02: cmp     si,1
        jz      Show03

        inc     cx

Show03: add     bx,cx
        cmp     bx,79
        jbe     Show04

        sub     cx,ax
        xor     ax,ax
        call    NewLine
        mov     bx,cx

Show04: or      ax,ax
        jz      Show05

        push    dx
        mov     ah,2
        mov     dl,20h              ;space
        int     21h
        pop     dx

Show05: mov     ah,9
        int     21h
        cmp     si,1
        jz      Show06

        mov     dl,2Ch              ;comma
        mov     ah,2
        int     21h

Show06: add     di,2
        dec     si
        jnz     Show01

        call    NewLine
        pop     cx
        pop     si
        dec     cx
        jnz     Show00

;------------------------------------------

;wait for a key press

        mov     dx,offset s$AnyKey  ;'Press Any Key To Exit'
        call    StrOut
        mov     ah,1                ;wait for key
        int     16h                 ;BIOS keyboard call
        mov     ah,0                ;get keyboard character
        int     16h                 ;BIOS keyboard call

;------------------------------------------

;terminate program

        mov     ax,4C00h            ;terminate, return = 0
        int     21h                 ;DOS function call

_main   ENDP

;********************************************************************

NewLine PROC    NEAR
;
;Display a Carriage Return/Line Feed
;
;Call With: Nothing
;
;  Returns: Nothing
;
;     Uses: all registers are preserved
;
;------------------------------------------

        push    dx
        push    ax
        mov     dx,offset s$CrLf
        mov     ah,9
        int     21h
        pop     ax
        pop     dx
        ret

NewLine ENDP

;********************************************************************

StrOut  PROC    NEAR
;
;Display a String Followed by Carriage Return/Line Feed
;
;Call With: DX = address of string to display
;
;  Returns: Nothing
;
;     Uses: all registers are preserved
;
;------------------------------------------

        push    ax
        mov     ah,9
        int     21h
        call    NewLine
        pop     ax
        ret

StrOut  ENDP

;********************************************************************

PrInp   PROC    NEAR
;
;Get 16-Bit Unsigned User Input With Prompt And Retry Error Handling
;
;Call With: DX = address of prompt string to display
;
;  Returns: DX = unsigned integer entered by user
;
;Also Uses: all other registers are preserved
;           the "InpBuf" buffer is used to hold the string
;
;------------------------------------------

        push    ax
        push    bx
        push    cx
        push    si

PrInp0: push    dx
        mov     ah,9
        int     21h
        mov     dx,offset InpBuf
        mov     ah,0Ah
        int     21h
        call    NewLine
        xor     dx,dx
        mov     cl,InpBuf.UsedBytes
        mov     ch,dl
        mov     si,offset InpBuf.Buffer
        mov     ah,dl
        jcxz    PrInp2

PrInp1: lodsb
        xor     al,30h
        cmp     al,9
        ja      PrInp4

        cmp     dx,6553
        ja      PrInp5

        shl     dx,1
        mov     bx,dx
        shl     dx,1
        shl     dx,1
        add     dx,bx
        add     dx,ax
        jc      PrInp5

        loop    PrInp1

        or      dx,dx
        jnz     PrInp6

PrInp2: mov     dx,offset s$ErrMsg1 ;'Zero Not Allowed'

PrInp3: call    StrOut
        pop     dx
        jmp     PrInp0

PrInp4: mov     dx,offset s$ErrMsg2 ;'Non-Numeric Character in String'
        jmp     PrInp3

PrInp5: mov     dx,offset s$ErrMsg3 ;'Value Too Large'
        jmp     PrInp3

PrInp6: pop     ax                  ;discard string pointer
        pop     si                  ;restore registers
        pop     cx
        pop     bx
        pop     ax
        ret

PrInp   ENDP

;********************************************************************

UnsDec  PROC    NEAR
;
;Convert a 16-Bit Unsigned Integer to ASCII Decimal String
;
;Call With: AX = unsigned binary integer
;
;  Returns: DX = address of '$'-terminated ASCII decimal string
;           CX = length of ASCII decimal string
;
;Also Uses: all other registers are preserved
;           the "OutBuf" buffer is used to hold the string
;
;------------------------------------------

        push    di
        push    bx
        push    ax
        mov     di,offset OutBuf+sizeof OutBuf-1
        mov     bx,10               ;BX = divisor
        push    di

UnDec0: xor     dx,dx               ;zero the high word
        dec     di                  ;point to next string byte
        div     bx                  ;divide DX:AX by 10
        or      dl,30h              ;set ASCII bits in remainder
        mov     [di],dl             ;store the numeric character
        or      ax,ax               ;any quotient left ?
        jnz     UnDec0              ;loop until done

        pop     cx
        mov     dx,di               ;DX = address of first char
        sub     cx,di               ;CX = string length
        pop     ax
        pop     bx
        pop     di
        ret

UnsDec  ENDP

;####################################################################

        END     _main

